!(function (a) {
  a.fn.captcha = function (b) {
    function getCaptchaStyleName() {
      let styleName;
      if ((styleName = element.data('captchastylename'))) {
        return styleName;
      }
      if ((styleName = element.data('stylename'))) {
        return styleName;
      }
      throw new Error(
        'The data-captchastylename attribute is not found or its value is not set.'
      );
    }

    function getHtml() {
      return a.ajax({
        method: 'GET',
        url: b.captchaEndpoint,
        data: { get: 'html', c: p },
      });
    }

    function validateUnsafe(captchaCode, onSuccess) {
      const instance = getInstance();
      a.ajax({
        method: 'GET',
        url: instance.validationUrl,
        data: { i: captchaCode },
        success(isCorrect) {
          onSuccess(isCorrect);
        },
      });
    }

    function registerUserInputBlurValidation() {
      const instance = getInstance();
      a(`#${instance.options.userInputID}`).on('blur', function () {
        const captchaCode = this.value.trim();
        if (captchaCode.length !== 0) {
          const userInputID = this;
          validateUnsafe(captchaCode, function (isCorrect) {
            if (!isCorrect) {
              instance.reloadImage();
            }
            a(userInputID).trigger('validatecaptcha', [isCorrect]);
          });
        }
      });
    }

    function getScript(url, options) {
      options = a.extend(
        {
          dataType: 'script',
          cache: false,
          url: url,
        },
        options || {}
      );
      return a.ajax(options);
    }

    function loadScriptIncludes() {
      const captchaId = a(`#BDC_VCID_${p}`).val();
      getScript(`${b.captchaEndpoint}?get=script-include&c=${p}&t=${captchaId}&cs=2`).done(
        function () {
          setTimeout(onLoadScriptsSuccess, 200);
        }
      );
    }

    function useUserInputBlurValidation() {
      const instance = getInstance();
      return (
        a(`#${instance.options.userInputID}`).attr('data-correct-captcha') !== undefined
      );
    }

    function onLoadScriptsSuccess() {
      if (useUserInputBlurValidation()) {
        registerUserInputBlurValidation();
      }
    }

    function getInstance() {
      return window.botdetect !== undefined
        ? window.botdetect.getInstanceByStyleName(p)
        : null;
    }

    function getCaptchaEndpointHandler() {
      const parts = b.captchaEndpoint.split('/');
      return parts[parts.length - 1];
    }

    function getBackendBaseUrl(captchaEndpointHandler) {
      const lastIndex = b.captchaEndpoint.lastIndexOf(captchaEndpointHandler);
      return b.captchaEndpoint.substring(0, lastIndex);
    }

    function changeRelativeToAbsoluteUrls(originCaptchaHtml) {
      const captchaEndpointHandler = getCaptchaEndpointHandler();
      const backendUrl = getBackendBaseUrl(captchaEndpointHandler);
      originCaptchaHtml = originCaptchaHtml.replace(/<script.*<\/script>/g, '');
      const relativeUrls = originCaptchaHtml.match(/(src|href)=\"([^"]+)\"/g);
      let relativeUrl;
      let absoluteUrl;
      let changedCaptchaHtml = originCaptchaHtml;
      for (let i = 0; i < relativeUrls.length; i++) {
        relativeUrl = relativeUrls[i]
          .slice(0, -1)
          .replace(/src=\"|href=\"/, '');
        const relativeUrlPrefixPattern = new RegExp(`.*${captchaEndpointHandler}`);
        absoluteUrl = relativeUrl.replace(
          relativeUrlPrefixPattern,
          backendUrl + captchaEndpointHandler
        );
        changedCaptchaHtml = changedCaptchaHtml.replace(
          relativeUrl,
          absoluteUrl
        );
      }
      return changedCaptchaHtml;
    }

    function displayHtml() {
      getHtml(b.captchaEndpoint, p).done(function (captchaHtml) {
        captchaHtml = changeRelativeToAbsoluteUrls(captchaHtml);
        element.html(captchaHtml);
        loadScriptIncludes();
      });
    }

    const element = this;
    if (element.length === 0) {
      throw new Error('Captcha html element cound not be found.');
    }
    if (!b || !b.captchaEndpoint) {
      throw new Error('The captchaEndpoint setting is required.');
    }
    b.captchaEndpoint = b.captchaEndpoint.replace(/\/+$/g, '');
    const p = getCaptchaStyleName();
    return (
      (element.init = function () {
        return displayHtml(), element;
      }),
      (element.getCaptchaId = function () {
        return getInstance().captchaId;
      }),
      (element.getCaptchaCode = function () {
        return getInstance().userInput.value;
      }),
      (element.getUserEnteredCaptchaCode = function () {
        return element.getCaptchaCode();
      }),
      (element.reloadImage = function () {
        getInstance().reloadImage();
      }),
      (element.validateUnsafe = function (callback) {
        const instance = getInstance();
        validateUnsafe(
          a.trim(a(`#${instance.options.userInputID}`).val()),
          function (isHuman) {
            callback(isHuman);
            if (!useUserInputBlurValidation() && !isHuman) {
              instance.reloadImage();
            }
          }
        );
      }),
      element.init()
    );
  };
})(jQuery);
